///////////////////////////////////////////////////////////////////////////////
//
// Meltice 2.x source code.
//
///////////////////////////////////////////////////////////////////////////////
// DI_ARDLL.CPP = DoIt group / for common ARchiver project's DLL functions.
///////////////////////////////////////////////////////////////////////////////
// $Id: DI_ARDLL.CPP,v 1.17 2001/06/16 16:35:35 ardry Exp $ 
///////////////////////////////////////////////////////////////////////////////

#include 	<windows.h>
#include	<shlobj.h>
#include	"di_doit.h"
#include	"u_util.h"

#include	"c_config.h" // for only debug option "NoArDll" and "WriteLog"

//-----------------------------------------------------------------------------
// for Common Archiver Project DLL settings...
#define		SUPPORT_DLL_NUM 	8

HINSTANCE  hInstDll[SUPPORT_DLL_NUM+1];

//dll chk...
static char dllname[SUPPORT_DLL_NUM+1][16] = 
{	"",
	"UNLHA32.DLL",
	"UNZIP32.DLL",
	"ZIP32J.DLL",
	"CAB32.DLL",
	"TAR32.DLL",
	"BGA32.DLL","YZ1.DLL",
	"JACK32.DLL"
};

static char getver_api[SUPPORT_DLL_NUM+1][24] = 
{
	"",
	"UnlhaGetVersion",
	"UnZipGetVersion", 
	"ZipGetVersion",
	"CabGetVersion",
	"TarGetVersion",
	"BgaGetVersion",
	"Yz1GetVersion",
	"JackGetVersion"
};

//archive chk...
static char chkarc_api[SUPPORT_DLL_NUM+1][24] = 
{
	"",
	"UnlhaCheckArchive",
	"UnZipCheckArchive",
	"",						//Dummy (ZIP32J.DLL can not unpack archive file!)
	"CabCheckArchive", 
	"TarCheckArchive", 
	"BgaCheckArchive", 
	"Yz1CheckArchive", 
	"JackCheckArchive"
};

static int chkarc_mode[SUPPORT_DLL_NUM+1] = {0, 4, 0, 0, 1, 0, 0, 0, 0};


//DLL ConfigDialog...
static char cfgdlg_api[SUPPORT_DLL_NUM+1][24] = {
	"",
	"UnlhaConfigDialog",
	"UnZipConfigDialog",
	"ZipConfigDialog",
	"CabConfigDialog",
	"TarConfigDialog",
	"BgaConfigDialog",
	"Yz1ConfigDialog",
	"JackConfigDialog"
};


//open_archive...
static char opnarc_api[SUPPORT_DLL_NUM+1][24] = 
{
	"",
	"UnlhaOpenArchive",
	"UnZipOpenArchive",
	"",						//Dummy (ZIP32J.DLL can not unpack archive file!)
	"CabOpenArchive", 
	"TarOpenArchive", 
	"BgaOpenArchive", 
	"Yz1OpenArchive", 
	"JackOpenArchive"
};

static char clsarc_api[SUPPORT_DLL_NUM+1][24] = 
{
	"",
	"UnlhaCloseArchive",
	"UnZipCloseArchive",
	"",
	"CabCloseArchive", 
	"TarCloseArchive", 
	"BgaCloseArchive", 
	"Yz1CloseArchive", 
	"JackCloseArchive"
};

static char find1st_api[SUPPORT_DLL_NUM+1][24] = 
{
	"",
	"UnlhaFindFirst",
	"UnZipFindFirst",
	"",
	"CabFindFirst", 
	"TarFindFirst", 
	"BgaFindFirst", 
	"Yz1FindFirst", 
	"JackFindFirst"
};

static char findnext_api[SUPPORT_DLL_NUM+1][24] =
{
	"",
	"UnlhaFindNext",
	"UnZipFindNext",
	"",
	"CabFindNext", 
	"TarFindNext", 
	"BgaFindNext", 
	"Yz1FindNext", 
	"JackFindNext"
};

static char getfilename_api[SUPPORT_DLL_NUM+1][24] = 
{
	"",
	"UnlhaGetFileName",
	"UnZipGetFileName",
	"",
	"CabGetFileName",
	"TarGetFileName", 
	"BgaGetFileName", 
	"Yz1GetFileName", 
	""
};

// processing...
static char arcrun_api[SUPPORT_DLL_NUM+1][12] = 
{
	"",
	"Unlha",
	"UnZip",
	"", 
	"Cab", 
	"Tar", 
	"Bga", 
	"Yz1", 
	"JackCombine"
};

static char arcrun_melt_cmd[SUPPORT_DLL_NUM+1][24] = {
	"",
	"x -jf0 -a1 ",
	"-o ", 
	"", 
	"-x -o ", 
	"-x -f ", 
	"x -a -o -r ", 
	"x -y ", 
	""
};



// load Common Archiver's Project DLL
//   return true  -> DLL load success!
//   return false -> DLL load failed.
static bool load_dll(int mode)
{
	if(mode == 0) return false;
	if(get_noardll(0) == true) return false;

	hInstDll[mode] =  LoadLibrary( dllname[mode] );
	if(hInstDll[mode] == NULL) return false;
	return true;
}

// Call archiver DLL config dialog.
//   hDlg     <- HWND hDlg.
//   mode     <- MODE_ZIP, etc...
bool config_dialog(const HWND hDlg, int mode)
{
	typedef bool (WINAPI *PCNFDLGFUNC)(const HWND,LPSTR,const int);
	static char dummybuf[1024];

	if( load_dll(mode) == true){
		PCNFDLGFUNC CfgDlg = (PCNFDLGFUNC)GetProcAddress(hInstDll[mode], cfgdlg_api[mode]);
		if(CfgDlg != NULL) // API function ready...
		{
			return (*CfgDlg)(hDlg,dummybuf, 1);
		}
	}
	return false;
}


// check archive.
//   filename <- input filename for check.
//   mode     <- MODE_ZIP, etc...
//   return true  -> archive file.
//   return false -> etc file.
static bool check_archive(const char * filename, int mode)
{
	typedef bool (WINAPI *PCHKARCFUNC)(LPCSTR,const int);

	if( load_dll(mode) == true){
		PCHKARCFUNC ChkArc = (PCHKARCFUNC)GetProcAddress(hInstDll[mode], chkarc_api[mode]);
		if(ChkArc != NULL) // API function ready...
		{
			return (*ChkArc)(filename,chkarc_mode[mode]);
		}
	}
	return false;
}

// Common Archiver's Project DLL versions.
// Ver.1.34 -> return 134.
int ardll_get_version(int mode)
{
	typedef WORD (WINAPI *PGETVERFUNC)();

	if( load_dll(mode) == true){
		PGETVERFUNC GetVer = (PGETVERFUNC)GetProcAddress(hInstDll[mode], getver_api[mode]);
		if(GetVer != NULL) // API function ready...
		{
			return (int)(*GetVer)();
		}
	}
	return 0;
}


int ardll_check_archive(const char *filename)
{

	//BugFix: TAR32.DLL v0.53 & 0-1byte file check -> Integer Divide by Zero.
	//(0-2byte file isn't archive file!)
	HANDLE chkarcfil = CreateFile( filename, GENERIC_READ, FILE_SHARE_READ,
		NULL, OPEN_EXISTING, NULL, NULL );
	if(GetFileSize(chkarcfil, NULL) <= 2) { CloseHandle(chkarcfil); return 0; }
	CloseHandle(chkarcfil);
	
	//Check archive by Common archiver project's DLL 
	if( check_archive(filename,MODE_JAK) == true) return MODE_JAK;
	if( check_archive(filename,MODE_YZ1) != false) return MODE_YZ1; 
							//YZ1.DLL return 0 and 2 when archive is yz1 format!
	if( check_archive(filename,MODE_TAR) == true) return MODE_TAR;
	if( check_archive(filename,MODE_BGA) == true) return MODE_BGA;
	if( check_archive(filename,MODE_CAB) == true) return MODE_CAB;
	if( check_archive(filename,MODE_ZIP) == true) return MODE_ZIP;
	if( check_archive(filename,MODE_LZH) == true) return MODE_LZH;
	return 0;
}

// unpack file from archive file.
//  hDlg      <- hWnd.
//  filename  <- archive file.
//  outputdir <- destdir.
//  mode      <- select unpack engine.
int ardll_melt(const HWND hDlg, const char *filename,const char *outputdir,int mode)
{
	int ret;
	typedef BOOL  (WINAPI *PARCRUNFUNC)(const HWND, LPCSTR,LPSTR, const DWORD);
	typedef DWORD (WINAPI *PARCCMBFUNC)(LPCSTR, LPCSTR ); // for JACK32.DLL
	
	if( load_dll(mode) == true){
		if(mode != MODE_JAK)
		{
			PARCRUNFUNC ArcRun = (PARCRUNFUNC)GetProcAddress(hInstDll[mode], arcrun_api[mode]);
			if(ArcRun != NULL) // API function ready...
			{
				char cmdbuf[MAX_PATH * 4];
				wsprintf(cmdbuf,"%s \"%s\" \"%s\"", arcrun_melt_cmd[mode], filename, outputdir);
				if(mode==MODE_TAR)
				{
					wsprintf(cmdbuf,"%s \"%s\" -o \"%s\"", arcrun_melt_cmd[mode], filename, outputdir);
				}
				ret = (*ArcRun)(hDlg,cmdbuf,NULL,0);
#ifdef	MELTICE
				if(get_writelog()==true)
				{
					char *logbuf = new char[MAX_PATH * 5];
					wsprintf(logbuf, "%s : [%s cmd] <%s>",filename, dllname[mode], cmdbuf);
					OutputLog("MELTICE.LOG",logbuf);

					if(ret == 0)
						wsprintf(logbuf, "%s : [OK] Meltice succeeded in processing of this file.",filename);
					else
						wsprintf(logbuf, "%s : [NG] Meltice failed in processing of this file. (0x%04x)",filename,ret);
					OutputLog("MELTICE.LOG",logbuf);
					delete [] logbuf;
				}
#endif
			return ret;
			}
		}else{	//for JACK32.DLL
			PARCCMBFUNC ArcCmb = (PARCCMBFUNC)GetProcAddress(hInstDll[mode], arcrun_api[mode]);
			if(ArcCmb != NULL) // API function ready...
			{
				ret= (*ArcCmb)(filename,outputdir);

#ifdef	MELTICE
				if(get_writelog()==true)
				{
					char *logbuf = new char[MAX_PATH * 5];
					wsprintf(logbuf, "%s : [%s API] JackCombine(\"%s\",\"%s\");",filename, dllname[mode], filename,outputdir); //$$$
					OutputLog("MELTICE.LOG",logbuf);

					if(ret == 0)
					{
						wsprintf(logbuf, "%s : [OK] Meltice succeeded in processing of this file.",filename);
						OutputLog("MELTICE.LOG",logbuf);
					}else{
						wsprintf(logbuf, "%s : [NG] Meltice failed in processing of this file. (0x%x)",filename,ret);
						OutputLog("MELTICE.LOG",logbuf);
					}
					delete [] logbuf;
				}
#endif
			return ret;
			}
		}
	}
	return false;

}

//for no double make folder function.
// singledir <- if singledir return true then save singledir path.
// return true -> singledir.
// return false -> not single dir.
bool ardll_check_archive_in_singledir
	(const char *filename, char *singledir, int sdlen)
{
	int mode = ardll_check_archive(filename);
	if(mode == MODE_JAK) return true; //Jack must be SingleFile!

	if( mode != MODE_NOT_SUPPORT && load_dll(mode) == true )
	{
		//for OpenArchive API
		typedef HARC (WINAPI *POPNARCFUNC)(const HWND, LPCSTR, const DWORD);
		//for CloseArchive API
		typedef int (WINAPI *PCLSARCFUNC)(HARC);
		//for FindFirst API
		typedef int (WINAPI *PFF_ARCFUNC)(HARC, LPCSTR,INDIVIDUALINFO FAR *);
		//for FindNext API
		typedef int (WINAPI *PFN_ARCFUNC)(HARC,INDIVIDUALINFO FAR *);
		//for GetFileName API
		typedef int (WINAPI *PGFNARCFUNC)(HARC,LPSTR,const int);

		POPNARCFUNC OpnArc = (POPNARCFUNC)GetProcAddress(hInstDll[mode], opnarc_api[mode]);
		PCLSARCFUNC ClsArc = (PCLSARCFUNC)GetProcAddress(hInstDll[mode], clsarc_api[mode]);
		PFF_ARCFUNC FF_Arc = (PFF_ARCFUNC)GetProcAddress(hInstDll[mode], find1st_api[mode]);
		PFN_ARCFUNC FN_Arc = (PFN_ARCFUNC)GetProcAddress(hInstDll[mode], findnext_api[mode]);
		PGFNARCFUNC GfnArc = (PGFNARCFUNC)GetProcAddress(hInstDll[mode], getfilename_api[mode]);

		if(	OpnArc != NULL &&
			ClsArc != NULL &&
			FF_Arc != NULL &&
			FN_Arc != NULL &&
			GfnArc != NULL ) // API function ready...
		{
			char *buff = new char[MAX_PATH*10];   //buffer.
			char *chkbuf = new char[MAX_PATH*10]; //for check.
			bool chk = false; //chk=true  ->Singlefolder false -> not single.
			HARC arc;
			unsigned int iCnt;

			//OpenArchive
			if( mode == MODE_LZH )
			{
				arc = (*OpnArc)(NULL,filename,M_REGARDLESS_INIT_FILE|M_RECOVERY_ON);
			}else{
				arc = (*OpnArc)(NULL,filename,M_REGARDLESS_INIT_FILE);
			}

			INDIVIDUALINFO ivinf;
			(*FF_Arc)(arc,"*.*",&ivinf); // FindFirst
			(*GfnArc)(arc,buff,MAX_PATH*10); //GetFileName
			iCnt = 1; //FileCount = 1.
			get1stDirName(buff); // hogehoge/foobar/abcde.txt -> hogehoge
			lstrcpyn(chkbuf, buff,MAX_PATH*10); //chkbuf set standard for check
			while( (*FN_Arc)(arc,&ivinf) == 0 ){ // FindNext
				(*GfnArc)(arc,buff,MAX_PATH*10);//GetFileName
				iCnt++;
				get1stDirName(buff);
				if(lstrcmp(buff, chkbuf)==0)
				{
					lstrcpyn(singledir, buff,sdlen);
					chk = true;
				}else{
					chk=false;
					break;
				}
			}
			
			if(iCnt == 1) //for single file in archive file.
			{
				(*FF_Arc)(arc,"*.*",&ivinf); // FindFirst
				(*GfnArc)(arc,buff,MAX_PATH*10); //GetFileName
				lstrcpyn(chkbuf, buff,MAX_PATH*10); //chkbuf = Original Filename.
				get1stDirName(buff); // SingleFile and SingleDir : hogehoge/abcde.txt -> hogehoge
									 // SingleFile and NotSingleDir : abcde.txt -> abcde.txt
				if(lstrcmp(buff, chkbuf)!=0) // buff <> chkbuf -> singlefile &&singledir.
				{
					lstrcpyn(singledir, buff,sdlen);
					chk = true;
				}
			}

			(*ClsArc)(arc); // Close Archive

			delete [] buff;
			delete [] chkbuf;

			return chk;
		}
	}

	return false;

}


